#include "automaton.h"
using namespace std;

namespace AlgorithMan
{


    State::State(int x, int y, string Label)
    {
        this->x = x;
        this->y = y;
        this->Label = Label;
    }

    State::~State()
    {
        while(!IncomingTransitions.empty())
        {
            Transition* t = *IncomingTransitions.begin();
            delete t;
        }

        while(!Transitions.empty())
        {
            Transition* t = *Transitions.begin();
            delete t;
        }
    }

    void State::AddTransition(Transition* transition)
    {
        transition->setFrom(this);
    }

    State* State::Clone(map<const State*, State*>& clones) const
    {
        // has been cloned already
        if(clones.find(this) != clones.end())
            return clones[this];

        State* result = new State(x,y,Label);
        result->InitialState = InitialState;
        result->FinalState = FinalState;

        clones[this] = result;
        for(set<Transition*>::const_iterator i = Transitions.begin(); i != Transitions.end(); i++)
            result->AddTransition( (*i)->Clone(clones) );
        return result;
    }

    void State::SetInitial()
    {
        this->InitialState = true;
    }
    bool State::IsInitial()
    {
        return this->InitialState;
    }
    void State::UnsetInitial()
    {
        this->InitialState = false;
    }

    void State::SetFinal()
    {
        this->FinalState = true;
    }
    bool State::IsFinal()
    {
        return this->FinalState;
    }
    void State::UnsetFinal()
    {
        this->FinalState = false;
    }




    Transition::Transition(State* from, State* to)
    {
        this->from = NULL;
        this->to = NULL;
        this->setFrom(from);
        this->setTo(to);
    }

    Transition::~Transition()
    {
        this->setFrom(NULL);
        this->setTo(NULL);
    }

    void Transition::setFrom(State* from)
    {
        if(this->from != NULL)
            this->from->Transitions.erase(this);
        this->from = from;
        if(this->from != NULL)
            this->from->Transitions.insert(this);
    }

    void Transition::setTo(State* to)
    {
        if(this->to != NULL)
            this->to->IncomingTransitions.erase(this);
        this->to = to;
        if(this->to != NULL)
            this->to->IncomingTransitions.insert(this);
    }

    Transition* Transition::Clone(map<const State*, State*>& clones) const
    {
        return new Transition(from->Clone(clones), to->Clone(clones));
    }

    
    DeltaTransition::DeltaTransition(char character, State* from, State* to)
    :   Transition(from, to)
    {
        this->character = character;
    }

    DeltaTransition::~DeltaTransition()
    {
    }

    Transition* DeltaTransition::Clone(map<const State*, State*>& clones) const
    {
        return new DeltaTransition(character, from->Clone(clones), to->Clone(clones));
    }




    Automaton::Automaton()
    {
    }

    Automaton::Automaton(const Automaton& A) // Copy Constructor
    {
        Insert(A);
    }


    void Automaton::Insert(const Automaton& A)
    {
        map<const State*,State*> stateclones;
        for(set<State*>::const_iterator i = A.Q.begin(); i != A.Q.end(); i++)
            this->AddState((*i)->Clone(stateclones));

        for(set<char>::const_iterator i = A.Sigma.begin(); i != A.Sigma.end(); i++)
            this->Sigma.insert(*i);
    }


    Automaton::Automaton(Automaton&& A) // Move Constructor
    {
        Sigma = A.Sigma;
        Q = A.Q;
        //A.Q.clear();
    }

    Automaton* Automaton::Clone() const
    {
        Automaton* result = new Automaton();
        result->Sigma = this->Sigma;

        map<const State*,State*> stateclones;
        for(set<State*>::const_iterator i = Q.begin(); i != Q.end(); i++)
            result->AddState((*i)->Clone(stateclones));

        return result;
    }

    Automaton::~Automaton()
    {
        for(set<State*>::iterator i = Q.begin(); i != Q.end(); i++)
            delete *i;
    }

    void Automaton::AddState(State* state)
    {
        Q.insert(state);
    }

    void Automaton::RemoveState(State* state)
    {
        Q.erase(state);
    }



    // ---------------------------------------- Dealing with Epsilon-Transitions



    bool Automaton::Accepts(istream& s) const
    {
        char lastread;
        if(Deterministic())
        {
            State* q = NULL;
            for(set<State*>::const_iterator i = Q.begin(); i != Q.end() && q == NULL; i++)
                if((*i)->IsInitial())
                    q = *i;

            if(q == NULL)
                return false; // automata without initial states accept nothing

            while(s.peek() != -1)
            {
                s.get(lastread);

                bool transitionfound = false;
                for(set<Transition*>::iterator i = q->Transitions.begin(); i != q->Transitions.end() && !transitionfound; i++)
                {
                    DeltaTransition* d = static_cast<DeltaTransition*>(*i);
                    if(d->character == lastread)
                    {
                        q = d->to;
                        transitionfound = true;
                    }
                }
                if(!transitionfound)
                    return false;
            }

            return q->IsFinal();
        }
        else // non-deterministic
        {
            set<State*> states;
            for(set<State*>::const_iterator i = Q.begin(); i != Q.end(); i++)
                if((*i)->IsInitial())
                    states.insert(*i);
            EpsilonHull(states);

            while(s.peek() != -1)
            {
                s.get(lastread);

                set<State*> nextstates;
                for(set<State*>::iterator i = states.begin(); i != states.end(); i++)
                {
                    for(set<Transition*>::iterator j = (*i)->Transitions.begin(); j != (*i)->Transitions.end(); j++)
                    {
                        DeltaTransition* d = dynamic_cast<DeltaTransition*>(*i);
                        if(d != NULL && d->character == lastread)
                            nextstates.insert(d->to);
                    }
                }
                EpsilonHull(nextstates);
                states = nextstates;
            }

            for(set<State*>::iterator i = states.begin(); i != states.end(); i++)
                if((*i)->IsFinal())
                    return true;
            return false;
        }
    }






    // ---------------------------------------- Dealing with Epsilon-Transitions



    void Automaton::EpsilonHull(set<State*>& states) const
    {
        queue<const State*> q;
        for(set<State*>::iterator i = states.begin(); i != states.end(); i++)
            q.push(*i);

        while(!q.empty())
        {
            const State* front = q.front();

            for(set<Transition*>::iterator i = front->Transitions.begin(); i != front->Transitions.end(); i++)
            {
                if(dynamic_cast<DeltaTransition*>(*i) != NULL) // not an epsilon transition
                    continue;

                if(states.find((*i)->to) == states.end())
                {
                    states.insert((*i)->to);
                    q.push((*i)->to);
                }
            }

            q.pop();
        }
    }


    // --------------------------------------------------------- Determinization


    bool Automaton::Deterministic() const
    {
        int InitialStates = 0;
        for(set<State*>::const_iterator i = Q.begin(); i != Q.end(); i++)
        {
            // deterministic automata can only have one initial state
            if((*i)->IsInitial())
                if(++InitialStates > 1)
                    return false;

            set<char> SeenCharacters;
            for(set<Transition*>::iterator j = (*i)->Transitions.begin(); j != (*i)->Transitions.end(); j++)
            {
                DeltaTransition* d = dynamic_cast<DeltaTransition*>(*j);
                if(d == NULL) // its an epsilon transition
                    return false;
                if(SeenCharacters.find(d->character) != SeenCharacters.end())
                    return false;
                SeenCharacters.insert(d->character);
            }
        }
        return true;
    }


/*

    Automaton Automaton::Determinized() const
    {
        if(Deterministic())
            return *this;
        return EpsilonElimination().EpsilonFreeDeterminization();
    }

*/

    // ------------------------------------------ Construction of other Automata


/*
    Automaton Automaton::Complement() const
    {
        if(!Deterministic())
            return Determinized().Complement();

        Automaton result(*this);
        // invert set of final states
        for(set<State*>::iterator i = result.Q.begin(); i != result.Q.end(); i++)
            if((*i)->IsFinal())
                (*i)->UnsetFinal();
            else
                (*i)->SetFinal();

        return result;
    }



*/


/*



    // ------------------------------------------------ Old Data-Structure stuff










    // Powerset Construction
    Automaton Automaton::EpsilonFreeDeterminization() const
    {
        vector<bool> current(Q.size(),false);
        vector<bool> next(Q.size(),false);

        vector< TreeNode* > queue(1);

        current[q0] = true;
        queue[0] = root->Insert(&current);
        queue[0]->id = 0;
        int q_front = 0, q_end = 1;

        while(q_front < q_end)  // find all required states
        {
            queue[q_front]->ReconstructStateSet(&current);

            for(set<char>::const_iterator it = alphabet.begin(); it != alphabet.end(); it++)
            {
                next = vector<bool>(n,false);
                for(int k = 0; k < n; k++)
                {
                    if(current[k])
                    {
                        map<char, set<int> >::const_iterator succ = delta[k].find(*it);
                        if(succ != delta[k].end())
                            for(set<int>::const_iterator l = succ->second.begin(); l != succ->second.end(); ++l)
                                next[*l] = true;
                    }
                }
                // now "next" contains all symbol-successors of the current stateset
                nextnode = root->Insert(&next);
                if(nextnode->id == -1) // if next stateset is new => enqueue
                {
                    nextnode->id = q_end++;
                    queue.push_back(nextnode);
                }
            }
            q_front++;
        } // done searching for required states

        // now construct the automata from the gathered informations
        Automaton result(q_end);
        result.q0 = 0;
        for(int k = 0; k < q_end; k++)
        {
            queue[k]->ReconstructStateSet(&current);
            for(int l = 0; l < n; l++)
            {
                if(current[l] && F[l])
                {
                    result.F[k] = true;
                    break;
                }
            }
            for(int l = 0; l < n; l++)
                if(current[l])
                    result.Labels[k] += Labels[l] + ",";

            for(set<char>::iterator it = alphabet.begin(); it != alphabet.end(); it++)
            {
                next = vector<bool>(n,false);
                for(int q = 0; q < n; q++)
                {
                    if(current[q])
                    {
                        map<char, set<int> >::const_iterator succ = delta[q].find(*it);
                        if(succ != delta[q].end())
                            for(set<int>::const_iterator l = succ->second.begin(); l != succ->second.end(); ++l)
                                next[*l] = true;
                    }
                }
                // now "next" contains all symbol-successors of the current stateset
                nextnode = root->Insert(&next);
                result.AddTransition(k, *it, nextnode->id);
            }
        }

        delete root;
        return result;
    }




    // --------------------------------------------------------------------------




    Automaton Automaton::Reverse() const
    {
        Automaton result(n);
        result.x = x;
        result.y = y;
        result.Labels = Labels;
        result.alphabet = alphabet;

        int Fsize=0;
        for(int i = 0; i < n && Fsize < 2; i++)
            if(F[i]) {
                result.q0 = i;
                Fsize++;
            }
        if (Fsize != 1) {
            result.q0 = result.AddState();
            for(int i = 0; i < n; i++) {
                if(F[i])
                    result.AddEpsilonTransition(result.q0,i);
            }
        }

        int i2 = 0;
        for(vector<map<char,set<int> > >::const_iterator i = delta.begin(); i != delta.end(); ++i, ++i2) {
            for(map<char,set<int> >::const_iterator a = i->begin(); a != i->end(); ++a) {
                for(set<int>::const_iterator j = a->second.begin(); j != a->second.end(); ++j) {
                    result.AddTransition(*j, a->first, i2);
                }
            }
        }

        i2 = 0;
        for(vector<set<int> >::const_iterator i = epsilon.begin(); i != epsilon.end(); ++i, ++i2) {
            for(set<int>::const_iterator j = i->begin(); j != i->end(); ++j) {
                result.AddEpsilonTransition(*j,i2);
            }
        }

        result.F[q0] = 1;
        result.deterministic = -1;
        return result;
    }


    Automaton Automaton::Combine(const Automaton& A, bool AnorB, bool onlyA, bool onlyB, bool AandB, int BfinalID) const {
        vector< vector<int> > ResultStateIdx(n, vector<int>(A.n, -1));
        ResultStateIdx[q0][A.q0] = 0;
        int NextStateIdx = 1;

        vector< pair<int,int> > queue(n*A.n);
        queue[0] = pair<int,int>(q0, A.q0);
        int q_front = 0, q_end = 1;
        while(q_front < q_end) {
            int i = queue[q_front].first;
            int j = queue[q_front].second;

            for(map<char,set<int> >::const_iterator is = delta[i].begin(); is != delta[i].end(); ++is) {
                map<char,set<int> >::const_iterator js = A.delta[j].find(is->first);
                if(js == A.delta[j].end())
                    continue;
                for(set<int>::const_iterator i1 = is->second.begin(); i1 != is->second.end(); ++i1)
                    for(set<int>::const_iterator j1 = js->second.begin(); j1 != js->second.end(); ++j1)
                        if(ResultStateIdx[*i1][*j1] == -1) {
                            ResultStateIdx[*i1][*j1] = NextStateIdx++;
                            queue[q_end++] = pair<int,int>(*i1,*j1);
                        }
            }
            for(set<int>::const_iterator i2 = epsilon[i].begin(); i2 != epsilon[i].end(); ++i2)
                for(set<int>::const_iterator j2 = A.epsilon[j].begin(); j2 != A.epsilon[j].end(); ++j2)
                    if(ResultStateIdx[*i2][*j2] == -1) {
                        ResultStateIdx[*i2][*j2] = NextStateIdx++;
                        queue[q_end++] = pair<int,int>(*i2,*j2);
                    }

            q_front++;
        }

        Automaton result(q_front); // now queue contains all required states
        for(int i = 0; i < q_front; ++i) {
            for(map<char,set<int> >::const_iterator is = delta[queue[i].first].begin(); is != delta[queue[i].first].end(); ++is) {
                map<char,set<int> >::const_iterator js = A.delta[queue[i].second].find(is->first);
                if(js == A.delta[queue[i].second].end())
                    continue;

                for(set<int>::const_iterator i1 = is->second.begin(); i1 != is->second.end(); ++i1)
                    for(set<int>::const_iterator j1 = js->second.begin(); j1 != js->second.end(); ++j1)
                        result.AddTransition(i, is->first, ResultStateIdx[*i1][*j1]);

            }

            for(set<int>::const_iterator ie = epsilon[queue[i].first].begin(); ie != epsilon[queue[i].first].end(); ++ie)
                for(set<int>::const_iterator je = A.epsilon[queue[i].second].begin(); je != A.epsilon[queue[i].second].end(); ++je)
                    result.AddEpsilonTransition(i ,ResultStateIdx[*ie][*je]);

            // the multiplications are there to enable the construction of
            // backtrack automata (for scanner generators)
            if(F[queue[i].first])
            {
                if(A.F[queue[i].second])
                {
                    result.F[i] = AandB*F[queue[i].first];
                }
                else
                {
                    result.F[i] = onlyA*F[queue[i].first];
                }
            }
            else
            {
                if(A.F[queue[i].second])
                {
                    result.F[i] = onlyB*BfinalID;
                }
                else
                {
                    result.F[i] = AnorB;
                }
            }

            result.Labels[i] = "(" + Labels[queue[i].first] + "," +
                               A.Labels[queue[i].second] + ")";
            result.x[i] = queue[i].first * 20 + 20;
            result.y[i] = queue[i].second * 20 + 20;
        }
        result.q0 = 0; // = (q0, A.q0)

        return result;
    }

    Automaton Automaton::Intersection(const Automaton& A) const
    {
        return Combine(A, 0, 0, 0, 1);
    }

    Automaton Automaton::operator+(const Automaton& A) const
    {
        return Combine(A, 0, 1, 1, 1);
    }

    Automaton Automaton::operator-(const Automaton& A) const
    {
        return Combine(A, 0, 1, 0, 0);
    }

    Automaton Automaton::operator*(const Automaton& A) const
    {
        Automaton result(*this, A);
        result.q0 = q0;
        for(int i = 0; i < n; i++)
            if(F[i])
            {
                result.AddEpsilonTransition(i, n + A.q0);
                result.F[i] = false;
            }
        return result;
    }

    bool Automaton::operator<=(const Automaton& A) const
    {
        return Intersection(A.Complement()).Empty();
    }

    bool Automaton::operator==(const Automaton& A) const
    {
        return Minimized().Isomorphic(A.Minimized());
    }

    bool Automaton::operator!=(const Automaton& A) const
    {
        return !operator==(A);
    }

    bool Automaton::operator<(const Automaton& A) const
    {
        return operator<=(A) && !operator==(A);
    }

    bool Automaton::operator>=(const Automaton& A) const
    {
        return A.operator<=(*this);
    }

    bool Automaton::operator>(const Automaton& A) const
    {
        return A.operator<(*this);
    }


    bool Automaton::Isomorphic(const Automaton& A) const
    {
        if(!Deterministic())
            return Determinized().Isomorphic(A);

        if(!A.Deterministic())
            return Isomorphic(A.Determinized());

        if(n != A.n)
            return false;
        vector<int> Isomorphism(n,-1);
        vector<int> IsomorphismInv(n,-1);
        Isomorphism[q0] = A.q0;
        IsomorphismInv[A.q0] = q0;

        vector<int> queue(n,-1);
        queue[0] = q0;
        int q_front=0, q_end=1;
        char symbol;

        while(q_front < q_end)
        {
            int i = queue[q_front];
            int j = Isomorphism[i];

            // iterate successors
            for(map<char, set<int> >::const_iterator it = delta[i].begin(); it != delta[i].end(); it++)
            {
                map<char, set<int> >::const_iterator jt = A.delta[j].find(it->first);
                if(jt == A.delta[j].end()) // i has a transition, that j has not
                    return false;

                // *(xt->second.begin()) is x's successor w.r.t. the symbol
                // it->first

                if(Isomorphism[*(it->second.begin())] == -1)
                {
                    if(IsomorphismInv[*(jt->second.begin())] == -1)
                    {
                        Isomorphism[*(it->second.begin())] = *(jt->second.begin());
                        IsomorphismInv[*(jt->second.begin())] = *(it->second.begin());
                        queue[q_end++] = *(it->second.begin());
                    }
                    else
                    {
                        // if IsomorphismInv[j's successor] != -1, then
                        // it can only be legal if
                        //    IsomorphismInv[j's successor]
                        //    == i's successor
                        // but then Isomorphism[i's successor] would be
                        // equal to j's successor, not -1
                        return false;
                    }
                }
                else
                {
                    // when there is already an isomorphism for i's successor,
                    // it must be j's successor, because otherwise
                    // i and j can not be isomorphic, because they have non-isomorphic
                    // successors
                    if(Isomorphism[*(it->second.begin())] != *(jt->second.begin()))
                        return false;
                    // however if j's successor is already known to be the isomorphism
                    // of i's successor, then nothing has to be done
                }
            }
            for(map<char, set<int> >::const_iterator jt = A.delta[j].begin(); jt != A.delta[j].end(); jt++)
            {
                if(delta[i].find(jt->first) == delta[i].end()) // j has a transition, that i has not
                    return false;
            }

            q_front++;
        }
        return true;
    }

    Automaton Automaton::Quotient(const Automaton& A) const
    {
        Automaton result(Minimized());
        Automaton MinA(A.Minimized());
        vector<int> resultF(result.n,0);
        int resultq0 = result.q0;
        for(int i = 0; i < result.n; i++)
        {
            result.q0 = i;
            if(!result.Intersection(MinA).Empty())
                resultF[i] = 1;
        }
        result.q0 = resultq0;
        result.F = resultF;
        return result;
    }

    Automaton Automaton::KleeneHull() const
    {
        Automaton result(*this);
        result.q0 = result.AddState(true);
        result.AddEpsilonTransition(n, q0);

        for(int i = 0; i < n; i++)
        {
            if(F[i])
            {
                result.F[i] = false;
                result.AddEpsilonTransition(i,n);
            }
        }
        return result;
    }

    Automaton Automaton::KleenePlus() const
    {
        Automaton result(*this);
        result.q0 = result.AddState(false);
        result.AddState(true); // new final state

        result.AddEpsilonTransition(n, q0);
        result.AddEpsilonTransition(n+1,n);

        for(int i = 0; i < n; i++)
        {
            if(F[i])
            {
                result.F[i] = false;
                result.AddEpsilonTransition(i,n+1);
            }
        }
        return result;
    }

    Automaton Automaton::ShuffleProduct(const Automaton& A) const
    {
        Automaton result(n*A.n);
        for(int i = 0; i < n; i++)
        {
            for(int j = 0; j < A.n; j++)
            {
                result.x[i*A.n+j] = 20*j + 20;
                result.y[i*A.n+j] = 20*i + 20;

                const map<char,set<int> >* pi = &delta[i];
                for(map<char,set<int> >::const_iterator k = pi->begin(); k != pi->end(); k++)
                    for(set<int>::const_iterator l = k->second.begin(); l != k->second.end(); l++)
                        result.AddTransition(i*A.n+j, k->first, (*l)*A.n+j);
                const set<int>* epsi = &epsilon[i];
                for(set<int>::const_iterator cur_eps = epsi->begin(); cur_eps != epsi->end(); cur_eps++)
                    result.AddEpsilonTransition(i*A.n+j, (*cur_eps)*A.n+j);

                const map<char,set<int> >* pj = &delta[j];
                for(map<char,set<int> >::const_iterator k = pj->begin(); k != pj->end(); k++)
                    for(set<int>::const_iterator l = k->second.begin(); l != k->second.end(); l++)
                        result.AddTransition(i*A.n+j, k->first, i*A.n+(*l));
                const set<int>* epsj = &epsilon[j];
                for(set<int>::const_iterator cur_eps = epsj->begin(); cur_eps != epsj->end(); cur_eps++)
                    result.AddEpsilonTransition(i*A.n+j, i*A.n+(*cur_eps));
            }
        }
        return result;
    }

    Automaton Automaton::SynchronizedProduct(const Automaton& A) const
    {
        Automaton ACopy(A), TCopy(*this);
        for(set<char>::const_iterator i = ACopy.alphabet.begin(); i != ACopy.alphabet.end(); i++)
            if(TCopy.alphabet.find(*i) == TCopy.alphabet.end())
            {
                TCopy.alphabet.insert(*i);
                for(int j = 0; j < n; j++)
                    TCopy.AddTransition(j, *i, j);
            }
        for(set<char>::const_iterator i = TCopy.alphabet.begin(); i != TCopy.alphabet.end(); i++)
            if(ACopy.alphabet.find(*i) == ACopy.alphabet.end())
            {
                ACopy.alphabet.insert(*i);
                for(int j = 0; j < A.n; j++)
                    ACopy.AddTransition(j, *i, j);
            }
        return ACopy.Intersection(TCopy);
    }

    // --------------------------------------------------------------------------



    Automaton Automaton::EpsilonElimination() const
    {
        for(vector<set<int> >::const_iterator i = epsilon.begin(); i != epsilon.end(); ++i)
            if(i->size() > 0)
                goto has_epsilon;
        return *this;
        has_epsilon:

        Automaton result(*this);
        vector<bool> BeforeTransition;
        vector<bool> AfterTransition;

        for(int i = 0; i < n; i++)
        {
            BeforeTransition = vector<bool>(n,false);
            BeforeTransition[i] = true;
            EpsilonHull(BeforeTransition);
            if(F[i] == 0)
            {
                for(int j = 0; j < n; j++)  // if a final state is in i's epsilon-hull,
                {
                    if(BeforeTransition[j] && (F[j]!=0))  // then i becomes final
                    {
                        result.F[i] = 1;
                        break;
                    }
                }
            }

            for(int j = 0; j < n; j++)
            {
                if(!BeforeTransition[j])
                    continue;
                for(map<char,set<int> >::const_iterator it = delta[j].begin(); it != delta[j].end(); ++it)
                {
                    AfterTransition = vector<bool>(n,false);
                    for(set<int>::const_iterator k = it->second.begin(); k != it->second.end(); ++k)
                        AfterTransition[*k] = true;
                    EpsilonHull(AfterTransition);
                    for(int k = 0; k < n; k++)
                        if(AfterTransition[k])
                            result.AddTransition(i, it->first, k);
                }
            }
        }

        result.ClearEpsilon();
        return result;
    }

    Automaton Automaton::Minimized() const
    {
        if(!ConstDeterministic())
            return Determinized().Minimized();

        vector<bool> reachable(n, false);
        vector<int> queue(n);
        queue[0] = q0;
        reachable[q0]=true;
        int queue_head=0;
        int queue_tail=1;
        while(queue_head < queue_tail)
        {
            const map<char,set<int> > *current = &delta[queue[queue_head++]];
            for(map<char,set<int> >::const_iterator i = current->begin(); i != current->end(); i++)
            {
                for(set<int>::const_iterator j = i->second.begin(); j != i->second.end(); j++)
                {
                    if(!reachable[*j])
                    {
                        reachable[*j] = true;
                        queue[queue_tail++] = *j;
                    }
                }
            }
            // minimization only works for DFAs, so there are no epsilon transitions
        }

        vector< vector <bool> > Distinguishable(n, vector<bool>(n,false));
        for(int i = 0; i < n; ++i)
            for(int j = i+1; j < n; ++j)
                if(F[i] != F[j])
                {
                    Distinguishable[i][j] = true;
                    Distinguishable[j][i] = true;
                }

        int Marks=0;
        bool Found; // determine which states are distinguishable
        do { // maximally n passes
            Found = false;
            for(int i = 0; i < n; ++i)
                for(int j = i+1; j < n; ++j)
                {
                    if(Distinguishable[i][j])
                        continue;
                    for(map<char,set<int> >::const_iterator is = delta[i].begin(); is != delta[i].end(); ++is)
                    {
                        map<char,set<int> >::const_iterator js = delta[j].find(is->first);
                        if(js != delta[j].end())
                        {
                            if(Distinguishable[*(is->second.begin())][*(js->second.begin())])
                            {
                                Distinguishable[i][j] = true;
                                Distinguishable[j][i] = true;
                                Found = true;
                                ++Marks;
                            }
                        }
                    }
                }
        } while(Found);
        if(2*Marks == (n*n)-n) // automata is already minimal
            return(*this);

        int EquivalenceClasses = 0;
        vector<int> EquivalenceClass(n,-1);
        for(int i = 0; i < n; ++i)
            if(reachable[i])
                if(EquivalenceClass[i] == -1)
                {
                    EquivalenceClass[i] = EquivalenceClasses++;
                    for(int j = i+1; j < n; ++j)
                        if(!Distinguishable[i][j])
                            EquivalenceClass[j] = EquivalenceClass[i];
                }

        Automaton result(EquivalenceClasses); // create resulting automata
        result.q0 = EquivalenceClass[q0];
        for(int i = 0; i < n; ++i)
            if(reachable[i])
                if(F[i])
                    result.F[EquivalenceClass[i]] = true;
        for(int i = 0; i < n; ++i)
            if(reachable[i])
                for(int j = 0; j < n; ++j)
                    if(reachable[j])
                        for(set<char>::const_iterator k = SymbolsOnTransitions[i][j].begin(); k != SymbolsOnTransitions[i][j].end(); ++k)
                            result.AddTransition(EquivalenceClass[i], *k, EquivalenceClass[j]);

        return result;
    }


    // --------------------------------------------------------------------------


    // this is the construction of a regular expression by state-elimination
    // note that it adds a new theoretical state <n> and the rest is handled like
    // <n> was the only final state and there are epsilon-transitions from all
    // final-states to <n>. this way the resulting regexp can only have the form
    // regexp[q0][q0]*regexp[q0][n]
    // because <n> has no outgoing transitions or loops, there is no need for
    // (regexp[q0][n]regexp[n][q0])* and regexp[n][n]*
    string Automaton::ToRegularExpression() const
    {
        vector< vector<string> > regexps(n+1, vector<string>(n+1));
        for(int i = 0; i < n; ++i)
        {
            for(int j = 0; j < n; ++j)
                if(SymbolsOnTransitions[i][j].size() > 0 || (HasEpsilon[i][j]&&i!=j))
                {
                    if(SymbolsOnTransitions[i][j].size() > 0)
                    {
                        regexps[i][j] = string(1,*(SymbolsOnTransitions[i][j].begin()));
                        set<char>::const_iterator k = SymbolsOnTransitions[i][j].begin();
                        for(++k; k != SymbolsOnTransitions[i][j].end(); ++k)
                            regexps[i][j] += "+" + string(1,*k);
                        if(HasEpsilon[i][j] && i!=j)
                            regexps[i][j] += "+\\varepsilon";
                    }
                    else
                    {
                        regexps[i][j] = "\\varepsilon";
                    }
                }
            if(F[i])
                regexps[i][n] = "\\varepsilon";
        }

        for(int i = 0; i < n; i++) // state i is eliminated (just a copy of it)
        {
            if(i == q0)
                continue;
            for(int j = 0; j < n; j++) // add regex for j->i->k to j->k
            {
                if(j <= i && j != q0)
                    continue;
                if(regexps[j][i] != "")
                {
                    string temp;
                    if(regexps[j][i] != "\\varepsilon")
                        temp = "(" + regexps[j][i] + ")";
                    if(regexps[i][i] != "")
                        temp += "("+regexps[i][i]+")^*";
                    for(int k = 0; k <= n; k++)
                    {
                        if(k <= i && k != q0)
                            continue;
                        if(regexps[i][k] != "")
                        {
                            string plus = regexps[j][k] == "" ? "" : "+";
                            if(regexps[i][k] != "\\varepsilon")
                                regexps[j][k] += plus + temp + "(" + regexps[i][k] + ")";
                            else
                                regexps[j][k] += plus + temp;
                        }
                    }
                }
            }
        }

        if(regexps[q0][n] == "")
            return "\\emptyset";
        if(regexps[q0][q0] == "" || regexps[q0][q0] == "\\varepsilon")
            return regexps[q0][n];
        else
            return "(" + regexps[q0][q0] + ")^*"
                 + (regexps[q0][n] == "\\varepsilon" ? "" : regexps[q0][n]);
    }


    // --------------------------------------------------------------------------


    bool Automaton::Empty() const
    {
        if(F[q0])
            return(false);
        vector<bool> visited(n,false);
        visited[q0] = true;
        vector<int> queue(n,-1);
        queue[0] = q0;
        int q_front=0, q_end=1;

        while(q_front < q_end)
        {
            // examine delta-successors of the head-element of the queue
            const map<char,set<int> >* cur_trans = &delta[queue[q_front]];
            for(map<char,set<int> >::const_iterator i = cur_trans->begin(); i != cur_trans->end(); ++i)
            {
                for(set<int>::const_iterator succ = i->second.begin(); succ != i->second.end(); ++succ)
                {
                    if(visited[*succ]) // successor was already examined
                        continue;
                    if(F[*succ]) // final state reachable => not empty
                        return false;
                    visited[*succ] = true; // enqueue successor
                    queue[q_end++] = *succ;
                }
            }

            // examine epsilon-successors of the head-element of the queue
            const set<int>* cur_eps = &epsilon[queue[q_front]];
            for(set<int>::const_iterator succ = cur_eps->begin(); succ != cur_eps->end(); ++succ)
            {
                if(visited[*succ]) // successor was already examined
                    continue;
                if(F[*succ]) // final state reachable => not empty
                    return false;
                visited[*succ] = true; // enqueue successor
                queue[q_end++] = *succ;
            }

            q_front++; // dequeue
        }
        return true;
    }


    bool Automaton::Universal() const
    {
        // universality for NFAs is hard => must be determinized, so
        // using complement is ok and no big runtime killer, although
        // complement uses the (exponential) Determinized() method
        return Complement().Empty();
    }


    bool Automaton::Finite() const
    {
        vector< vector<bool> > reachable(n, vector<bool>(n,false));
        for(int i = 0; i < n; i++)
        {
            for(int j = 0; j < n; j++)
            {
                if(SymbolsOnTransitions[i][j].size() > 0 || (HasEpsilon[i][j]&&i!=j))
                    reachable[i][j] = true;
            }
        }

        // Warshalls Algorithm
        for(int k = 0; k < n; ++k)
            for(int i = 0; i < n; ++i)
                if(reachable[i][k])
                    for(int j = 0; j < n; j++)
                        if(reachable[k][j])
                            reachable[i][j] = true;

        for(int i = 0; i < n; i++)
        {
            if(!reachable[q0][i])
                continue;
            for(int j = 0; j < n; j++)
            {
                if(!reachable[i][j] || !reachable[j][i])
                    continue;
                for(int k = 0; k < n; k++)
                {
                    if(F[k] && reachable[i][k])
                        return false;
                }
            }
        }

        return true;
    }

    // ------------------------------------------------------------------- export


    void Automaton::LaTeX(ostream &os, int h, int w) const {
        os << "% \\usepackage{gastex}\n";
        os << "% \\usepackage[usenames]{color}\n";
        os << "% get gastex.sty e.g. from http://www.lsv.ens-cachan.fr/~gastin/gastex/gastex.sty\n\n";

        float maxx=x[0], minx=x[0], maxy=y[0], miny=y[0];
        for( int i = 1; i < n; ++i)
        {
            if(maxx < x[i]) maxx = x[i];
            if(minx > x[i]) minx = x[i];
            if(maxy < y[i]) maxy = y[i];
            if(miny > y[i]) miny = y[i];
        }
        float scale = min(w/(maxx-minx), h/(maxy-miny));

        os << "\\begin{picture}("<<w<<","<<h<<")(0,"<<-h<<")\n";
        for( int i = 0; i < n; ++i)                                     // states
        {
            os << "\t\\node[linecolor=Black,fillcolor=White";
            if(i == q0 || F[i] != 0)
            {
                os << ",Nmarks=";
                if(i == q0)
                    os << "i";
                if(F[i] != 0)
                    os << "r";
            }
            os << "](q" << i << ")(" << (int)((x[i]-minx)*scale) << ","
               << -(int)((y[i]-miny)*scale) << "){" << Labels[i] << "}\n";
        }
        os << "\n";

        for(int i = 0; i < n; ++i)                                 // transitions
        {
            for(int j = 0; j < n; ++j)
            {
                if(SymbolsOnTransitions[i][j].size() > 0 || HasEpsilon[i][j])
                {
                    if(i != j)
                        os << "\t\\drawedge[linecolor=Black,curvedepth=2.0](q"<<i<<",q"<<j<<"){";
                    else
                        os << "\t\\drawloop[linecolor=Black,loopdiam=4,loopangle=90](q"<<i<<"){";

                    if(SymbolsOnTransitions[i][j].size() > 0)
                    {
                        set<char>::const_iterator k = SymbolsOnTransitions[i][j].begin();
                        os << *k;
                        for(++k; k != SymbolsOnTransitions[i][j].end(); ++k)
                            os << "," << *k;
                        if(HasEpsilon[i][j])
                            os << ",$\\varepsilon$";
                    }
                    else
                    {
                        os << "$\\varepsilon$";
                    }
                    os << "}\n";
                }
            }
            os << "\n";
        }
        os << "\\end{picture}\n";
    }

    string Automaton::LaTeX(int h, int w) const
    {
        stringstream s;
        LaTeX(s,h,w);
        return(s.str());
    }



    void Automaton::CCode(ostream& os) const
    {
        if(ConstDeterministic())
        {
            os << "bool accepts(istream& s) {\n";
            os << "\tchar lastread;\n";
            os << "\tgoto q" << q0 << ";\n\n";
            for(int i = 0; i < n; ++i) {
                os << "\tq" << i << ": //" << Labels[i] << "\n";
                os << "\t\ts.get(lastread);\n";
                os << "\t\tif(s.bad())\n\t\t\treturn " << (F[i]?"true":"false") << ";\n";
                os << "\t\tswitch(lastread) {\n";
                for(int j = 0; j < n; ++j) {
                    if(SymbolsOnTransitions[i][j].size() <= 0)
                        continue;
                    set<char>::const_iterator k = SymbolsOnTransitions[i][j].begin();
                    os << "\t\t\tcase \'" << *k << "\':";
                    for(++k; k != SymbolsOnTransitions[i][j].end(); ++k)
                        os << " case \'" << *k << "\':";
                    os << "\n\t\t\t\tgoto q" << j << ";\n";
                }
                os << "\t\t\tdefault: return false;\n";
                os << "\t\t}\n\n";
            }
            os << "}\n";
        }
        else  // nondeterministic
        {
            os << "bool accepts(istream& is) {\n";
            os << "\tint n = "<<n<<";\n";
            os << "\tchar lastread;\n";
            os << "\tbool *states = new(bool[n]);\n";
            os << "\tbool *nextstates = new(bool[n]);\n";
            os << "\tfor(int i = 0; i < n; i++) {\n";
            os << "\t\tstates[i] = false;\n";
            os << "\t}\n";
            vector<bool> states(n,false);
            states[q0]=true;
            EpsilonHull(states);
            for(int i = 0; i < n; i++)
                if(states[i])
                    os << "\tstates["<<i<<"] = true;\n";
            os << "\n";

            os << "\twhile(true) {\n";
            os << "\t\tfor(int i = 0; i < n; i++) {\n";
            os << "\t\t\tnextstates[i] = false;\n";
            os << "\t\t}\n";
            os << "\t\tis.get(lastread);\n";
            os << "\t\tif(is.bad())\n\t\t\tbreak;\n\n";

            for(int i = 0; i < n; i++)
            {
                os << "\t\tif(states["<<i<<"]) {\n";
                os << "\t\t\tswitch(lastread) {\n";
                vector<bool> epshull(n,false);
                epshull[i] = true;
                EpsilonHull(epshull);
                for(set<char>::const_iterator symb = alphabet.begin(); symb != alphabet.end(); ++symb)
                {
                    states = vector<bool>(n,false);
                    int numstates = 0;
                    for(int l = 0; l < n; l++)
                    {
                        if(!epshull[l])
                            continue;
                        map<char,set<int> >::const_iterator succ = delta[l].find(*symb);
                        if(succ == delta[l].end())
                            continue;

                        for(set<int>::const_iterator s = succ->second.begin(); s != succ->second.end(); ++s)
                        {
                            states[*s] = true;
                            numstates++;
                        }
                    }
                    if(numstates > 0)
                    {
                        os << "\t\t\t\tcase \'" << *symb << "\':\n";
                        EpsilonHull(states);
                        for(int k = 0; k < n; k++)
                            if(states[k])
                                os << "\t\t\t\t\tnextstates["<<k<<"] = true;\n";
                        os << "\t\t\t\t\tbreak;\n";
                    }
                }
                os << "\t\t\t}\n";
                os << "\t\t}\n";
            }

            os << "\n\t\tbool* temp = states;\n";
            os << "\t\tstates = nextstates;\n";
            os << "\t\tnextstates = temp;\n";
            os << "\t}\n\n";
            os << "\tbool result = false;\n";
            for(int i = 0; i < n; i++)
                if(F[i])
                    os << "\tif(states["<<i<<"]) result = true;\n";
            os << "\tdelete[](states);\n";
            os << "\tdelete[](nextstates);\n";
            os << "\treturn result;\n";
            os << "}\n";
        }
    }

    string Automaton::CCode() const
    {
        stringstream s;
        CCode(s);
        return(s.str());
    }


    void Automaton::MyhillNerode(ostream& os) const
    {
        Automaton temp(Minimized());
        temp.F = vector<int>(temp.n,0);

        vector<string> Representative(temp.n);
        vector<bool> visited(temp.n,false);
        visited[temp.q0] = true;
        vector<int> queue(temp.n);
        queue[0] = temp.q0;
        int q_front=0, q_end=1;

        os << "% you need to put\n";
        os << "%    \\usepackage{stmaryrd}\n";
        os << "%    \\usepackage{amsmath}\n";
        os << "%    \\usepackage{amssymb}\n";
        os << "% to the beginning of the Document\n\n";
        os << "\\begin{eqnarray*}\n";
        os << "\t\\llbracket \\varepsilon \\rrbracket_{\\equiv_L} & = & L(";
        temp.F[temp.q0] = 1;
        os << temp.ToRegularExpression() << ")";
        temp.F[temp.q0] = 0;

        while(q_front < q_end)
        {
            const map<char,set<int> >* front_delta = &temp.delta[queue[q_front]];
            for(map<char,set<int> >::const_iterator succ = front_delta->begin(); succ != front_delta->end(); ++succ)
            {
                int i = *(succ->second.begin()); // symb->second has exactly 1 member, because of the minimization
                if(visited[i])
                    continue;
                visited[i] = true;
                queue[q_end++] = i;

                Representative[i] = Representative[queue[q_front]] + string(1, succ->first);
                os << "\\\\\n\t\\llbracket "<<Representative[i]<<" \\rrbracket_{\\equiv_L} & = & L(";
                temp.F[i] = 1;
                os << temp.ToRegularExpression() << ")";
                temp.F[i] = 0;
            }
            q_front++;
        }
        os << "\n\\end{eqnarray*}\n";
        os << "$\\lvert {\\Sigma^*} \\diagup _{\\equiv_L}\\rvert = " << temp.n << "$\n";
    }

    string Automaton::MyhillNerode() const
    {
        stringstream s;
        MyhillNerode(s);
        return(s.str());
    }
*/
}

